"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("@babel/types");
const util_1 = require("./util");
exports.default = () => {
    const defaultOptions = {
        isRelease: true,
        var: {},
        imports: {
            ignoreLibraries: ['react'],
            ignoreFilenames: '',
            remove: true,
            customImports: [],
        },
        styles: {
            ignoreFilenames: '',
            remove: true,
        },
        propTypes: {
            ignoreFilenames: '',
            remove: true,
            onlyProduction: true,
        }
    };
    let globalOptions;
    const reDevVar = /^(console|logger|debug)$/;
    return {
        name: "remove-unuse-code",
        visitor: {
            Program(programPath, state) {
                if (!globalOptions) {
                    const options = state.opts;
                    globalOptions = Object.assign({}, defaultOptions, options);
                    (0, util_1.normalizeOptions)(globalOptions);
                }
            },
            DebuggerStatement(path, state) {
                if ((0, util_1.isRelease)(state)) {
                    path.remove();
                }
            },
            Identifier(path, state) {
                if ((0, util_1.isRelease)(state) && reDevVar.test(path.node.name)) {
                    path.getStatementParent().remove();
                }
            },
            IfStatement(path, state) {
                const { node } = path;
                let cr = (0, util_1.check)(node.test, state);
                if (cr === undefined)
                    return;
                let result = cr ? node.consequent : node.alternate;
                if (!result) {
                    path.remove();
                }
                else if ((0, types_1.isBlockStatement)(result)) {
                    path.replaceWithMultiple(result.body);
                }
                else {
                    path.replaceWith(result);
                }
            },
            ConditionalExpression(path, state) {
                const { node } = path;
                let cr = (0, util_1.check)(node.test, state);
                if (cr === undefined) {
                    return;
                }
                path.replaceWith(cr ? node.consequent : node.alternate);
            },
            ImportDeclaration(path) {
                var _a;
                const { node, scope } = path;
                const specifiers = node.specifiers;
                const sourceValue = node.source.value;
                const imports = globalOptions.imports;
                if ((0, util_1.isIgnore)(path.scope, globalOptions.imports.ignoreFilenames)) {
                    return;
                }
                if (imports.remove) {
                    for (let i = specifiers.length - 1; i >= 0; i--) {
                        const name = specifiers[i].local.name;
                        const binding = scope.getBinding(name);
                        if (((_a = binding === null || binding === void 0 ? void 0 : binding.referencePaths) === null || _a === void 0 ? void 0 : _a.length) === 0 && !imports.ignoreLibraries.includes(sourceValue)) {
                            (0, util_1.remove)(binding.path);
                        }
                        if (specifiers.length === 0) {
                            (0, util_1.remove)(path);
                        }
                    }
                }
                if (imports.customImports.length) {
                    const customImport = imports.customImports.find((imports) => imports.libraryName === sourceValue);
                    if (customImport && (0, types_1.isImportSpecifier)(specifiers[0])) {
                        const newImports = specifiers.map(specifier => {
                            var _a, _b, _c;
                            const localName = specifier.local.name;
                            const customUrl = (_b = (_a = customImport.customMapping) === null || _a === void 0 ? void 0 : _a[localName]) !== null && _b !== void 0 ? _b : `${(_c = customImport.libraryDirectory) !== null && _c !== void 0 ? _c : 'lib'}/${localName}`;
                            return ((0, types_1.importDeclaration)([(0, types_1.importDefaultSpecifier)(specifier.local)], (0, types_1.stringLiteral)(`${customImport.libraryName}/${customUrl}`)));
                        });
                        path.replaceWithMultiple(newImports);
                    }
                }
                if ((0, util_1.isProptypesRemove)(path, globalOptions)) {
                    if (sourceValue === 'prop-types') {
                        (0, util_1.remove)(path);
                    }
                }
            },
            ClassProperty(path) {
                var _a;
                if (!(0, util_1.isProptypesRemove)(path, globalOptions)) {
                    return;
                }
                const { node, scope } = path;
                if (((_a = node.key) === null || _a === void 0 ? void 0 : _a.name) === 'propTypes') {
                    const pathClassDeclaration = scope.path;
                    if ((0, util_1.isReactClass)(pathClassDeclaration.get('superClass'), scope)) {
                        (0, util_1.remove)(path);
                    }
                }
            },
            AssignmentExpression(path) {
                if (!(0, util_1.isProptypesRemove)(path, globalOptions)) {
                    return;
                }
                const { node, scope } = path;
                if (node.left.computed || !node.left.property || node.left.property.name !== 'propTypes') {
                    return;
                }
                const className = node.left.object.name;
                const binding = scope.getBinding(className);
                if (!binding) {
                    return;
                }
                if (binding.path.isClassDeclaration()) {
                    const superClass = binding.path.get('superClass');
                    if ((0, util_1.isReactClass)(superClass, scope)) {
                        (0, util_1.remove)(path);
                    }
                }
                else if ((0, util_1.isStatelessComponent)(binding.path)) {
                    (0, util_1.remove)(path);
                }
            },
        },
    };
};
//# sourceMappingURL=index.js.map